#!/usr/bin/env bash
# vim: set et sw=4 sts=4 :

# This hook makes Paludis store messages generated by an ebuild in a file.
# These stored messages are then displayed to the user after a merge. To enable
# this functionality, this script should be copied or symlinked into:
#
#     /usr/share/paludis/hooks/elog/
#     /usr/share/paludis/hooks/install_task_execute_pre/
#     /usr/share/paludis/hooks/install_task_execute_post/
#     /usr/share/paludis/hooks/uninstall_all_pre/
#     /usr/share/paludis/hooks/uninstall_all_post/
#     /usr/share/paludis/hooks/uninstall_fail/
#
# And optionally:
#
#     /usr/share/paludis/hooks/einfo/
#     /usr/share/paludis/hooks/ewarn/
#     /usr/share/paludis/hooks/eerror/
#
# You should ensure that it has execute permissions.

case "${EBUILD_PHASE}" in
    config|pretend|metadata) return
esac

source ${PALUDIS_ECHO_FUNCTIONS_DIR:-${PALUDIS_EBUILD_DIR}}/echo_functions.bash

logfile="${PALUDIS_HOOKS_TMPDIR:-${ROOT}/var/tmp/paludis}/elog.${PALUDIS_PID}"

case "${HOOK}" in
    elog)
    echo "L ${CATEGORY}/${!PALUDIS_NAME_VERSION_REVISION_VAR} ${MESSAGE}" >> "${logfile}"
    ;;

    einfo)
    echo "I ${CATEGORY}/${!PALUDIS_NAME_VERSION_REVISION_VAR} ${MESSAGE}" >> "${logfile}"
    ;;

    ewarn)
    echo "W ${CATEGORY}/${!PALUDIS_NAME_VERSION_REVISION_VAR} ${MESSAGE}" >> "${logfile}"
    ;;

    eerror)
    echo "E ${CATEGORY}/${!PALUDIS_NAME_VERSION_REVISION_VAR} ${MESSAGE}" >> "${logfile}"
    ;;

    install_task_execute_pre|install_all_pre|uninstall_all_pre)
    if [[ ${HOOK} = install_all_pre ]]; then
        paludis_ecmd \
            "ewarn" \
            "" \
            "`echo -ne " ${COLOUR_WARN}*${COLOUR_NORMAL} "`" \
            "Use of elog.bash in phase \"${HOOK}\" is deprecated, see comments at beginning of file for updated instructions"
    fi

    ${PRETEND} && exit 0

    rm -f "${logfile}"
    touch "${logfile}" || die "Couldn't create '${logfile}' for elog hooks"
    if [[ -n "${PALUDIS_REDUCED_GID}" ]] ; then
        chgrp "${PALUDIS_REDUCED_GID}" "${logfile}" || die "Couldn't chgrp '${logfile}' for elog hooks"
        chmod g+rw "${logfile}" || die "Couldn't chmod '${logfile}' for elog hooks"
    fi
    ;;

    install_all_post|uninstall_all_post|install_fail|uninstall_fail|clean_fail|install_task_execute_post)
    if [[ ${HOOK} = install_all_post || ${HOOK} = install_fail || ${HOOK} = clean_fail ]]; then
        paludis_ecmd \
            "ewarn" \
            "" \
            "`echo -ne " ${COLOUR_WARN}*${COLOUR_NORMAL} "`" \
            "Use of elog.bash in phase \"${HOOK}\" is deprecated, see comments at beginning of file for updated instructions"
    fi

    ${PRETEND} && exit 0

    echo
    old_pf=""
    while read line ; do
        line_status=$(echo "${line}" | cut -d' ' -f1 )
        line_pf=$(echo "${line}" | cut -d' ' -f2 )
        line_message=$(echo "${line}" | cut -d' ' -f3- )

        if [[ "${old_pf}" != "${line_pf}" ]] ; then
            echo
            echo "${COLOUR_GOOD}Summary of messages from ${line_pf}:${COLOUR_NORMAL}"
            echo
            old_pf="${line_pf}"
        fi

        case "${line_status}" in
            L|I)
            echo -ne " ${COLOUR_GOOD}*${COLOUR_NORMAL} "
            ;;

            W)
            echo -ne " ${COLOUR_WARN}*${COLOUR_NORMAL} "
            ;;

            E)
            echo -ne " ${COLOUR_BAD}*${COLOUR_NORMAL} "
            ;;
        esac
        echo "${line_message}"

    done <"${logfile}"

    rm -f "${logfile}"
    ;;

    *)
    paludis_ecmd \
        "ewarn" \
        "" \
        "`echo -ne " ${COLOUR_WARN}*${COLOUR_NORMAL} "`" \
        "elog.bash doesn't know what to do for HOOK=\"${HOOK}\""
    ;;
esac

